#!/bin/bash
########################################################################
# build_catch.sh
########################################################################

####################################
# Useful variables
prog=${0##*/}
thisdir=$(cd "$(dirname "${0}")" && pwd -P)
trace=maybe_xtrace # Could set to xtrace instead for unconditional.

####################################
# Useful functions.

# Report usage.
usage() {
  cat <<EOF
USAGE: ${prog} ${ssi_msg_config_usage:-<ssi-msg-config-options>} <product-dir> [tar]"
       ${prog} -h|--help

EOF
  (( ${1:-0} < 2 )) && return;
  cat <<EOF
OPTIONS

  -h

     This help.

${ssi_msg_config_options:-"  <ssi-msg-config-options>

    [Supply <product-dir> to enable more details]

"}\
ARGUMENTS

  <product-dir>

    The top-level UPS directory in which the source code distribution
    should be constructed.

  [tar]

    Generate a binary installation archive.

ENVIRONMENT

  DEBUG_BUILD
    Full command tracing for the build (VERBOSE=1)

  DEBUG_PATCHES
  VERBOSE_PATCHES
    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.

${ssi_msg_config_environment}
EOF
}

# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: ${*}\n" 1>&2; exit 1
}

# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    # We\'ve been asked for help: give as much as we can.
    usage 2; exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "${@}"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="${arg}"; break
  done
  if [ -z "${product_dir}" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> ${product_dir}\n$(usage)"
    exit 2
  elif ! { [ -d "${product_dir}/.upsfiles" ] && [ -w "${product_dir}" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: ${product_dir}"
  fi
  # Promising...
  local abs_product_dir="$(cd "${product_dir}" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: ${product_dir}"
  # Absolute is safer.
  product_dir="${abs_product_dir}"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="${product_dir}/ssibuildshims/${ssibuildshims_version}"
  [ -f "${ssibuildshims_topdir}/bin/ssi_functions" ] && \
    source "${ssibuildshims_topdir}/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "${product_dir}" && \
    setup ssibuildshims ${ssibuildshims_version} -z "${product_dir}" || \
    ssi_die "unable to find ssibuildshims ${ssibuildshims_version} in
${product_dir} and set it up with UPS."
}

# Ascertain whether we built and installed correctly.
is_built() {
set -x
  verify_in_filesystem "${@}" -d "${pkgdir}/include"
}

# If a distribution tarball is wanted and the installation is
# successful, then make it and exit.
maybe_make_distribution() {
  if [ "${maketar}" = tar ]; then
    is_built "$@" && make_basic_distribution && exit
  fi
}

########################################################################
# Main.
########################################################################

####################################
# Package name and version information.
package=catch
origpkgver=v2_11_3
pkgver=${origpkgver//-/_}
ssibuildshims_version=v2_00_00
pkgdotver=${origpkgver//_/.}; pkgdotver=${pkgdotver#v}
pkgtardir=Catch2-${pkgdotver}
pkgtarname=${pkgtardir}.tar.gz

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "${@}"

# Initialization and defaults.
(( ssi_trace = 1 ))

# Parse options.
while true; do
  case $1 in
    -h|--help) usage 2; exit 1;;
    --) shift; break;;
    -*) ssi_handle_config_options "${1}" || \
      ssi_die 2 "unrecognized option $1\n$(usage)";;
    *) shift; break # Swallow <product-dir>: dealt with.
  esac
  shift
done

# Environment overrides.
ssi_honor_environment_config

##################
# Remaining arguments.
maketar=${1}; shift

# Define globals.
define_basics NO_QUAL NO_DEBUG_SOURCE NO_FLAVOR NO_FQ_DIR || exit

# Possible short-circuit.
maybe_make_distribution -q

ssi_info "building ${package} for ${OS}-${plat} (flavor ${pkgflvr})"

####################################
# Ensure directories.
mkdir -p "${pkgdir}" || \
  ssi_die "failed to create installation directory ${pkgdir}"
# Remove any existing sourcedir to avoid patch conflicts.
rm -rf "${sourcedir}" && mkdir -p "${sourcedir}" || \
  ssi_die "failed to create source directory ${sourcedir}"
# Clean build.
rm -rf "${blddir}" && mkdir -p "${blddir}" || \
    ssi_die "failed to create build directory ${blddir}"
####################################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies.
${trace} fake_declare_product "${ssi_args[@]}" -x \
  "${product_dir}" ${package} ${pkgver} NO_FLAVOR || exit
####################################

####################################
# Build-only dependencies.

# Require CMake.
${trace} setup_cmake --latest || exit

# We can use GNU Make, but we'll take Ninja if it's available
${trace} setup_ninja --optional && (( want_ninja = 1 ))

# Need modern GCC for tests and to build examples.
if [[ "${OS1}" == Darwin ]] || \
  ! ver_greater 6.1.0 $(c++ -dumpversion); then
  ssi_info "using native cc/c++ for tests"
  export CXX=c++
  export CC=cc
elif setup_prod gcc --optional --latest={--ge,6.1.0} -- \
  -z "${product_dir}" -z "${PRODUCTS}"; then
	ssi_info "gcc ${GCC_VERSION} is available for use with tests"
	export CXX=g++
	export CC=gcc
else
	ssi_info "modern (GCC >= 6.1) is not available. Tests will be skipped."
fi
####################################

# Print active products.
echo
ups active
echo

cd "${sourcedir}" || ssi_die "Unable to cd into ${sourcedir}"

# Expand and patch source.
${trace} prepare_source "${tardir}/${pkgtarname}" \
  fix-examples-CMakeLists.patch || exit

[ -n "${DEBUG_PATCHES}" ] && exit 1

####################################
# Set all CMake options.

# Start with generator, if appropriate.
declare -a cmake_options
(( want_ninja )) && cmake_options+=(-GNinja)

# Verify language support for tests.
if ${trace} ${CXX} -xc++ -std=c++11 -c /dev/null >/dev/null 2>&1; then
  (( build_examples = 1 ))
else
  (( WANT_TESTS )) && \
    ssi_die "Catch2 requires C++11 or better for tests and examples"
fi

if (( WANT_TESTS )); then
  test_opt=ON
else
  test_opt=OFF
fi

cmake_options+=(-DCMAKE_INSTALL_PREFIX="${pkgdir}" \
  -DCATCH_BUILD_TESTING:BOOL=${test_opt} \
  -DCATCH_BUILD_EXTRA_TESTS:BOOL=${test_opt} \
  ${build_examples:+-DCATCH_BUILD_EXAMPLES:BOOL=ON} \
  -DCATCH_INSTALL_DOCS:BOOL=ON \
  -DCATCH_INSTALL_HELPERS:BOOL=ON)
####################################

####################################
# Ascertain and configure parallelism.
ncore=$(ncores)

# Default parallelism if not already set. 
[[ -n "${CMAKE_BUILD_PARALLEL_LEVEL}" ]] || \
  export CMAKE_BUILD_PARALLEL_LEVEL=${ncore}

[[ -n "${CTEST_PARALLEL_LEVEL}" ]] || \
  export CTEST_PARALLEL_LEVEL=${ncore}
####################################

cd "${blddir}" || ssi_die "unable to cd to build directory ${blddir}"

####################################
# Configure, build, and install.

# Configure.
${trace} cmake "${cmake_options[@]}" "${sourcedir}"/${pkgtardir} || \
  ssi_die "configure failed"

# Build.
${trace} cmake --build "${blddir}" ${DEBUG_BUILD:+--verbose} || \
  ssi_die "build failed"

# Test.
(( WANT_TESTS )) && \
  { ${trace} ctest || ssi_die "self-tests failed"; }

# Install.
${trace} cmake --install "${blddir}" || ssi_die "install failed"

####################################
# Installation verification and post-processing.

# Verify installation.
is_built -q || ssi_build_failed ${package}

ssi_info "finished building ${package} ${pkgver}"

# Put non-architecture-specific CMake files under share instead of lib.
if [[ -d "${pkgdir}/lib64" ]]; then
  libdir="lib64"
elif [[ -d "${pkgdir}/lib" ]]; then
  libdir="lib"
fi
if [[ -n "${libdir}" ]]; then
  ssi_info "relocating CMake files from ${libdir} -> share/"
  ${trace} mv "${pkgdir}"/{"${libdir}/cmake",share/} || \
    ssi_die "unable to relocate CMake files from ${libdir}/ to share/"
  rmdir "${pkgdir}/${libdir}" || ssi_die "unable to remove ${libdir}---not empty?"
fi

# Fix pkg-config file.
ssi_info "fix pkg-config file for relocatable package"
pkgcfg="${pkgdir}/share/pkgconfig/catch2.pc"
${trace} sed -Ei\~ -e 's&'"${pkgdir}"'(/|$)&\$\{pcfiledir\}/../..\1&' "${pkgcfg}" && \
  grep -qvFe "${product_dir}" "${pkgcfg}" >/dev/null 2>&1 && rm -f "${pkgcfg}~" || \
  ssi_die "unable to fix pkg-config"

# Declare and set up the installed package.
unsetup ${package} >/dev/null 2>&1
${trace} declare_product "${ssi_args[@]}" \
  -x "${product_dir}" ${package} ${pkgver} NO_FLAVOR || exit

ssi_info "${package} is installed at ${pkgdir}"

# This must be last
maybe_make_distribution
